#include "cpu.h"
#include "pic32_c.h"



/* Basic memory and pin initialisation. */

void init_memory(void)
{
    /*
    Configure RAM.
    See: http://microchipdeveloper.com/32bit:mx-arch-exceptions-processor-initialization
    */

    uint32_t config = REG(BMXCON);

    /* Set zero wait states for address setup. */

    config &= ~(1 << 6);        /* BMXCON<6> = BMXWSDRM = 0 */

    /* Set bus arbitration mode. */

    config &= ~0b111;
    config |= 0b010;            /* BMXCON<2:0> = BMXARB<2:0> = 2 */

    REG(BMXCON) = config;
}

void init_pins(void)
{
    /* DEVCFG0<2> also needs setting to 0 before the program is run. */

    CLR_REG(CFGCON, 1 << 3);    /* CFGCON<3> = JTAGEN = 0 */
}

void init_outputs(void)
{
    /* Remove analogue features from pins. */

    REG(ANSELA) = 0;
    REG(ANSELB) = 0;

    REG(TRISA) = 0;
    REG(TRISB) = 0;

    REG(PORTA) = 0;
    REG(PORTB) = 0;
}



/* Peripheral pin configuration. */

void config_uart(void)
{
    /* Map U1RX to RPB13. */

    REG(U1RXR) = 0b0011;        /* U1RXR<3:0> = 0011 (RPB13) */

    /* Map U1TX to RPB15. */

    REG(RPB15R) = 0b0001;       /* RPB15R<3:0> = 0001 (U1TX) */

    /* Set RPB13 to input. */

    SET_REG(TRISB, 1 << 13);
}

void lock_config(void)
{
    SET_REG(CFGCON, 1 << 13);   /* IOLOCK = 1 */

    /* Lock the configuration again. */

    REG(SYSKEY) = 0x33333333;
}

void unlock_config(void)
{
    /* Unlock the configuration register bits. */

    REG(SYSKEY) = 0;
    REG(SYSKEY) = 0xAA996655;
    REG(SYSKEY) = 0x556699AA;

    CLR_REG(CFGCON, 1 << 13);   /* IOLOCK = 0 */
}



/* Convenience operations. */

void interrupts_on(void)
{
    init_interrupts();
    enable_interrupts();
    handle_error_level();
}



/* Peripheral configuration. */

void init_uart(uint8_t pri, uint8_t sub)
{
    CLR_REG(U1MODE, 1 << 15);   /* U1MODE<15> = ON = 0 */
    REG(U1BRG) = 12;            /* U1BRG<15:0> = BRG = (FPB / (16 * baudrate)) - 1 = (24000000 / (16 * 115200)) - 1 = 12 */

    /* Disable interrupt and clear flag. */

    CLR_REG(IEC1, 1 << 8);      /* IEC1<8> = U1RIE = 0 */
    CLR_REG(IFS1, 1 << 8);      /* IFS1<8> = U1RIF = 0 */

    /* Set priorities: U1IP = pri; U1IS = sub */

    REG(IPC8) = (REG(IPC8) & ~0b11111) | ((pri & 0b111) << 2) | (sub & 0b11);

    /* Enable interrupt. */

    SET_REG(IEC1, 1 << 8);      /* IEC1<8> = U1RIE = 1 */

    /* Start UART. */

    SET_REG(U1STA, (1 << 12) | (1 << 10));  /* U1STA<12> = URXEN = 1; U1STA<10> = UTXEN = 1 */
    SET_REG(U1MODE, 1 << 15);               /* U1MODE<15> = ON = 1 */
}
