#include "pic32_c.h"
#include "init.h"

static void uart_write(char c)
{
    while (REG(U1STA) & (1 << 9));  /* UTXBF (buffer full) */

    REG(U1TXREG) = c;
}

static void bits(uint32_t reg)
{
    uint32_t mask;

    for (mask = (1 << 31); mask; mask >>= 1)
        if (REG(reg) & mask)
            uart_write('1');
        else
            uart_write('0');

    uart_write('\r');
    uart_write('\n');
}

static void blink(uint32_t delay, uint32_t port, uint32_t pins)
{
    uint32_t counter;

    /* Clear outputs (LED). */

    CLR_REG(port, pins);

    while (1)
    {
        counter = delay;

        while (counter--) __asm__("");	/* retain loop */

        /* Invert outputs (LED). */

        INV_REG(port, pins);
        bits(IFS1);
    }
}

void main(void)
{
    init_memory();
    init_pins();
    init_outputs();

    unlock_config();
    config_uart();
    lock_config();

    init_uart(7, 3);

    interrupts_on();

    blink(3 << 24, PORTA, 1 << 3);
}

void exception_handler(void)
{
    blink(3 << 12, PORTA, 1 << 3);
}

void interrupt_handler(void)
{
    /* Check for a UART receive interrupt condition (U1RIF). */

    if (!(REG(IFS1) & (1 << 8)))
        return;

    /* Write the received data back. */

    INV_REG(PORTA, 1 << 2);

    while (REG(U1STA) & 1)
        uart_write((char) REG(U1RXREG));

    /* Clear the UART interrupt condition. */

    CLR_REG(IFS1, 1 << 8);
}
